@@ -4,11 +4,8 @@
 #' 
 #' @param \dots Additional named elements passed to [Future()].
 #'
-#' @return An object of class `UniprocessFuture`.
-#'
-#' @seealso
-#' To evaluate an expression using "uniprocess future", see functions
-#' [uniprocess()].
+#' @return
+#' `UniprocessFuture()` returns an object of class `UniprocessFuture`.
 #'
 #' @export
 #' @name UniprocessFuture-class
@@ -16,21 +13,24 @@
 UniprocessFuture <- function(expr = NULL, substitute = TRUE, envir = parent.frame(), globals = TRUE, packages = NULL, lazy = FALSE, local = TRUE, ...) {
   if (substitute) expr <- substitute(expr)
 
-  if (lazy && !local && (!is.logical(globals) || globals)) {
-    stop("Non-supported use of lazy uniprocess futures: Whenever argument 'local' is FALSE, then argument 'globals' must also be FALSE. Lazy uniprocess future evaluation in the calling environment (local = FALSE) can only be done if global objects are resolved at the same time.")
-  }
-
-  ## Global objects?
-  gp <- getGlobalsAndPackages(expr, envir = envir, tweak = tweakExpression, globals = globals)
-  globals <- gp$globals
-  expr <- gp$expr
+  ## WORKAROUND: Skip scanning of globals if already done /HB 2021-01-18
+  if (!isTRUE(attr(globals, "already-done", exact = TRUE))) {
+    if (lazy && !local && (!is.logical(globals) || globals)) {
+      stop("Non-supported use of lazy uniprocess futures: Whenever argument 'local' is FALSE, then argument 'globals' must also be FALSE. Lazy uniprocess future evaluation in the calling environment (local = FALSE) can only be done if global objects are resolved at the same time.")
+    }
   
-  ## Record packages?
-  if (length(packages) > 0 || (length(gp$packages) > 0 && lazy)) {
-    packages <- unique(c(gp$packages, packages))
-  }
+    ## Global objects?
+    gp <- getGlobalsAndPackages(expr, envir = envir, tweak = tweakExpression, globals = globals)
+    globals <- gp$globals
+    expr <- gp$expr
   
-  gp <- NULL
+    ## Record packages?
+    if (length(packages) > 0 || (length(gp$packages) > 0 && lazy)) {
+      packages <- unique(c(gp$packages, packages))
+    }
+    
+    gp <- NULL
+  }
  
   future <- Future(expr = expr, substitute = FALSE, envir = envir, lazy = lazy, asynchronous = FALSE, local = local, globals = globals, packages = packages, ...)
   future <- structure(future, class = c("UniprocessFuture", class(future)))
@@ -54,16 +54,12 @@ run.UniprocessFuture <- function(future, ...) {
 
   expr <- getExpression(future)
   envir <- future$envir
+  if (future$local) envir <- new.env(parent = envir)
 
   ## Assign globals to separate "globals" enclosure environment?
   globals <- future$globals
   if (length(globals) > 0) {
-    if (future$local) {
-      envir <- new.env(parent = envir)
-    }
-    for (name in names(globals)) {
-      envir[[name]] <- globals[[name]]
-    }
+    envir <- assign_globals(envir, globals = globals, debug = debug)
   }
 
   ## Run future
@@ -73,13 +69,19 @@ run.UniprocessFuture <- function(future, ...) {
 
   if (debug) mdebugf("%s started (and completed)", class(future)[1])
 
+  ## WORKAROUND: Ditto warning by Future() is muffled for UniprocessFuture.
+  ## /HB 2022-04-27
+  if (!future$local) {
+    .Deprecated(msg = "Using 'local = FALSE' for a future is deprecated in future (>= 1.20.0) and will soon be defunct and produce an error.", package = .packageName)
+  }
+
   ## Always signal immediateCondition:s and as soon as possible.
   ## They will always be signaled if they exist.
   signalImmediateConditions(future)
 
   ## Signal conditions early, iff specified for the given future
   signalEarly(future, collect = FALSE)
-  
+
   invisible(future)
 }
 
@@ -123,13 +125,43 @@ resolved.UniprocessFuture <- function(x, ...) {
 }
 
 #' @export
-getExpression.UniprocessFuture <- function(future, immediateConditions = TRUE, ...) {
+getExpression.UniprocessFuture <- local({
+  tmpl_exit_rng_remove <- bquote_compile({
+    .(exit)
+    RNGkind(.(okind))
+    base::rm(list = ".Random.seed", envir = base::globalenv(), inherits = FALSE)
+  })
+  
+  tmpl_exit_rng_undo <- bquote_compile({
+    .(exit)
+    base::assign(".Random.seed", .(oseed), envir = base::globalenv(), inherits = FALSE)
+  })
+
+function(future, immediateConditions = TRUE, exit = NULL, ...) {
   ## Assert that no arguments but the first is passed by position
   assert_no_positional_args_but_first()
-  NextMethod(immediateConditions = immediateConditions)
+
+  ## Preserve RNG state?
+  oseed <- get_random_seed()
+  if (is.null(oseed)) {
+    okind <- RNGkind()[1]
+    exit <- bquote_apply(tmpl_exit_rng_remove)
+  } else {
+    exit <- bquote_apply(tmpl_exit_rng_undo)
+  }
+
+  NextMethod(immediateConditions = immediateConditions, exit = exit)
 }
+})
 
 
+#' @return
+#' `SequentialFuture()` returns an object of class `SequentialProcess`,
+#' which inherits from `UniprocessFuture`.
+#'
+#' @section Usage:
+#' To use 'sequential' futures, use `plan(sequential)`, cf. [sequential].
+#'
 #' @rdname UniprocessFuture-class
 #' @export
 SequentialFuture <- function(expr = NULL, envir = parent.frame(), substitute = TRUE, lazy = FALSE, globals = TRUE, local = TRUE, ...) {
@@ -137,3 +169,12 @@ SequentialFuture <- function(expr = NULL, envir = parent.frame(), substitute = T
   f <- UniprocessFuture(expr = expr, envir = envir, substitute = FALSE, lazy = lazy, globals = globals, local = local, ...)
   structure(f, class = c("SequentialFuture", class(f)))
 }
+
+
+#' @rdname UniprocessFuture-class
+#' @export
+TransparentFuture <- function(expr = NULL, envir = parent.frame(), substitute = TRUE, lazy = FALSE, globals = TRUE, local = FALSE, ...) {
+  if (substitute) expr <- substitute(expr)
+  f <- UniprocessFuture(expr = expr, envir = envir, substitute = FALSE, lazy = lazy, globals = globals, local = local, persistent = TRUE, ...)
+  structure(f, class = c("TransparentFuture", "SequentialFuture", class(f)))
+}
