@@ -1,5 +1,7 @@
 #' Retrieves global variables of an expression and their associated packages 
 #'
+#' @inheritParams globals::globalsOf
+#'
 #' @param expr An \R expression whose globals should be found.
 #' 
 #' @param envir The environment from which globals should be searched.
@@ -7,7 +9,7 @@
 #' @param tweak (optional) A function that takes an expression and returned a modified one.
 #' 
 #' @param globals (optional) a logical, a character vector, a named list, or a \link[globals]{Globals} object.  If TRUE, globals are identified by code inspection based on `expr` and `tweak` searching from environment `envir`.  If FALSE, no globals are used.  If a character vector, then globals are identified by lookup based their names `globals` searching from environment `envir`.  If a named list or a Globals object, the globals are used as is.
-#' 
+#'
 #' @param resolve If TRUE, any future that is a global variables (or part of one) is resolved and replaced by a "constant" future.
 #'
 #' @param persistent If TRUE, non-existing globals (= identified in expression but not found in memory) are always silently ignored and assumed to be existing in the evaluation environment.  If FALSE, non-existing globals are by default ignore, but may also trigger an informative error if option \option{future.globals.onMissing} in `"error"` (should only be used for troubleshooting).
@@ -28,12 +30,12 @@
 #' @export
 #'
 #' @keywords internal
-getGlobalsAndPackages <- function(expr, envir = parent.frame(), tweak = tweakExpression, globals = TRUE, resolve = getOption("future.globals.resolve", NULL), persistent = FALSE, maxSize = getOption("future.globals.maxSize", 500 * 1024 ^ 2), ...) {
+getGlobalsAndPackages <- function(expr, envir = parent.frame(), tweak = tweakExpression, globals = TRUE, locals = getOption("future.globals.globalsOf.locals", TRUE), resolve = getOption("future.globals.resolve", NULL), persistent = FALSE, maxSize = getOption("future.globals.maxSize", 500 * 1024 ^ 2), ...) {
   if (is.null(resolve)) {
     resolve <- FALSE
   } else {
     stop_if_not(is.logical(resolve), length(resolve) == 1L, !is.na(resolve))
-    .Deprecated(msg = sprintf("R option %s may only be used for troubleshooting. It must not be used in production since it changes how futures are evaluated and there is a greak risk that the results cannot be reproduced elsewhere: %s", sQuote("future.globals.resolve"), sQuote(resolve)), package = .packageName)
+    .Deprecated(msg = sprintf("R option %s may only be used for troubleshooting. It must not be used in production since it changes how futures are evaluated and there is a great risk that the results cannot be reproduced elsewhere: %s", sQuote("future.globals.resolve"), sQuote(resolve)), package = .packageName)
   }
   
   debug <- getOption("future.debug", FALSE)
@@ -55,7 +57,7 @@ getGlobalsAndPackages <- function(expr, envir = parent.frame(), tweak = tweakExp
     } else {
       globals.onMissing <- match.arg(globals.onMissing,
                                      choices = c("error", "ignore"))
-      .Deprecated(msg = sprintf("R option %s may only be used for troubleshooting. It must not be used in production since it changes how futures are evaluated and there is a greak risk that the results cannot be reproduced elsewhere: %s", sQuote("future.globals.onMissing"), sQuote(globals.onMissing)), package = .packageName)
+      .Deprecated(msg = sprintf("R option %s may only be used for troubleshooting. It must not be used in production since it changes how futures are evaluated and there is a great risk that the results cannot be reproduced elsewhere: %s", sQuote("future.globals.onMissing"), sQuote(globals.onMissing)), package = .packageName)
       mustExist <- is.element(globals.onMissing, "error")
     }
   }
@@ -76,7 +78,7 @@ getGlobalsAndPackages <- function(expr, envir = parent.frame(), tweak = tweakExp
       } else if (is.list(add)) {
         if (debug) mdebugf("- 'add' globals passed as-list: [%d] %s", length(add), hpaste(sQuote(names(add))))
       } else {
-        stop("Attribute 'add' of argument 'globals' must be either a character vector or a named list: ", mode(add))
+        stopf("Attribute 'add' of argument 'globals' must be either a character vector or a named list: %s", mode(add))
       }
       add <- as.FutureGlobals(add)
       stop_if_not(inherits(add, "FutureGlobals"))
@@ -95,12 +97,14 @@ getGlobalsAndPackages <- function(expr, envir = parent.frame(), tweak = tweakExp
       if (is.null(globals.method)) {
         globals.method <- "ordered"
       } else {
-        .Deprecated(msg = sprintf("R option %s may only be used for troubleshooting. It must not be used in production since it changes how futures are evaluated and there is a greak risk that the results cannot be reproduced elsewhere: %s", sQuote("future.globals.method"), sQuote(globals.method)), package = .packageName)
+        .Deprecated(msg = sprintf("R option %s may only be used for troubleshooting. It must not be used in production since it changes how futures are evaluated and there is a great risk that the results cannot be reproduced elsewhere: %s", sQuote("future.globals.method"), sQuote(globals.method)), package = .packageName)
       }
       
       globals <- globalsOf(
                    ## Passed to globals::findGlobals()
                    expr, envir = envir, substitute = FALSE, tweak = tweak,
+                   ## Requires globals (>= 0.14.0.9004); ignored otherwise
+                   locals = locals,
                    ## Passed to globals::findGlobals() via '...'
                    dotdotdot = "return",
                    method = globals.method,
@@ -138,7 +142,7 @@ getGlobalsAndPackages <- function(expr, envir = parent.frame(), tweak = tweakExp
   } else if (is.list(globals)) {
     if (debug) mdebugf("- globals passed as-list: [%d] %s", length(globals), hpaste(sQuote(names(globals))))
   } else {
-    stop("Argument 'globals' must be either a logical scalar or a character vector: ", mode(globals))
+    stopf("Argument 'globals' must be either a logical scalar or a character vector: %s", mode(globals))
   }
   ## Make sure to preserve 'resolved' attribute
   globals <- as.FutureGlobals(globals)
