@@ -1,5 +1,5 @@
 source("incl/start,load-only.R")
-
+options(future.debug=FALSE)
 message("*** plan() ...")
 
 cl <- future::makeClusterPSOCK(1L)
@@ -11,14 +11,14 @@ print(future::plan())
 future::plan(oplan)
 print(future::plan())
 
-message("*** Set strategy via future::plan(future::cluster, workers = cl, globals = FALSE)")
-oplan <- future::plan(future::cluster, workers = cl, globals = FALSE)
+message("*** Set strategy via future::plan(future::cluster, workers = cl)")
+oplan <- future::plan(future::cluster, workers = cl)
 print(future::plan())
 future::plan(oplan)
 print(future::plan())
 
-message("*** Set strategy via future::plan(future::cluster(workers = cl, globals = FALSE)")
-oplan <- future::plan(future::cluster(workers = cl, globals = FALSE))
+message("*** Set strategy via future::plan(future::cluster(workers = cl)")
+oplan <- future::plan(future::cluster(workers = cl))
 print(future::plan())
 future::plan(oplan)
 print(future::plan())
@@ -57,7 +57,6 @@ v <- value(f)
 print(v)
 stopifnot(v == 0)
 
-
 message("*** plan('sequential')")
 ## Setting strategy by name
 plan("sequential")
@@ -75,44 +74,50 @@ f <- future({ x <- 1 })
 print(value(f))
 stopifnot(x == 0)
 
-message("*** plan(sequential, local = FALSE)")
-plan(sequential, local = FALSE)
+message("*** plan(sequential, abc = 1)")
+plan(sequential, abc = 1, def = TRUE)
 fcn <- plan("next")
 print(fcn)
-stopifnot(formals(fcn)$local == FALSE)
-x <- 0
-f <- future({ x <- 1 })
-print(value(f))
-stopifnot(x == 1)
+stopifnot(formals(fcn)$abc == 1)
 
-message("*** plan(sequential, local = FALSE, abc = 1, def = TRUE)")
-plan(sequential, local = FALSE, abc = 1, def = TRUE)
-fcn <- plan("next")
-print(fcn)
-stopifnot(formals(fcn)$local == FALSE)
-
-message("*** plan(sequential(local = FALSE))")
+message("*** plan(sequential(abc = 1))")
 plan(cluster, workers = cl)
-plan(sequential(local = FALSE))
+plan(sequential(abc = 1))
 fcn <- plan("next")
 print(fcn)
-stopifnot(formals(fcn)$local == FALSE)
+stopifnot(formals(fcn)$abc == 1)
 
-message("*** plan(tweak(sequential, local = FALSE))")
+message("*** plan(tweak(sequential, abc = 1))")
 plan(cluster, workers = cl)
-plan(tweak(sequential, local = FALSE))
+plan(tweak(sequential, abc = 1))
 fcn <- plan("next")
 print(fcn)
-stopifnot(formals(fcn)$local == FALSE)
+stopifnot(formals(fcn)$abc == 1)
+
+message("*** plan(cluster, ..., rscript_startup = \"<code>\")")
+plan(cluster, workers = 1L, rscript_startup = "options(abc = 42L)")
+f <- future(getOption("abc"))
+v <- value(f)
+stopifnot(identical(v, 42L))
+plan(sequential)
+
+message("*** plan(cluster, ..., rscript_startup = <expr>)")
+plan(cluster, workers = 1L, rscript_startup = quote(options(abc = 42L)))
+f <- future(getOption("abc"))
+v <- value(f)
+print(v)
+stopifnot(identical(v, 42L))
+plan(sequential)
+
 
 
 message("*** old <- plan(new)")
 truth <- plan("list")
-old <- plan(cluster, workers = cl, globals = FALSE)
+old <- plan(cluster, workers = cl)
 stopifnot(identical(unclass(old), unclass(truth)))
 
-stack <- plan("list") ## stack == cluster(workers = cl, globals = FALSE)
-prev <- plan(old)     ## prev == sequential(local = FALSE)
+stack <- plan("list") ## stack == cluster(workers = cl)
+prev <- plan(old)     ## prev == sequential(abc = 1)
 stopifnot(identical(stack, prev))
 
 stack <- plan("list") ## curr == old
@@ -122,7 +127,7 @@ stopifnot(identical(stack, truth))
 message("*** %plan% 'sequential'")
 plan(cluster, workers = cl)
 x %<-% { a <- 1 } %plan% "sequential"
-stopifnot(identical(body(plan("next")), body(cluster)))
+stopifnot(inherits(plan("next"), "cluster"))
 
 message("*** %plan% sequential")
 plan(cluster, workers = cl)
@@ -134,16 +139,8 @@ f <- fun(1)
 stopifnot(inherits(f, "SequentialFuture"), !f$lazy, inherits(f, "SequentialFuture"))
 
 x %<-% { a <- 1 } %plan% sequential
-stopifnot(identical(body(plan("next")), body(cluster)))
+stopifnot(inherits(plan("next"), "cluster"))
 
-message("*** %plan% sequential(local = FALSE) ")
-plan(cluster, workers = cl)
-a <- 0
-x %<-% { a } %plan% sequential(local = FALSE)
-a <- 42
-print(x)
-stopifnot(x == 0)
-stopifnot(identical(body(plan("next")), body(cluster)))
 
 message("*** Nested futures with different plans")
 
@@ -224,6 +221,21 @@ print(plan())
 
 message("*** plan() w/ commands ... DONE")
 
+
+message("*** plan() - odds'n'ends ...")
+
+plan(sequential, split = FALSE)
+f <- future(42L)
+v <- value(f)
+stopifnot(v == 42L)
+stopifnot(
+  inherits(f$envir, "environment"),
+  identical(f$envir, globalenv())
+)
+
+message("*** plan() - odds'n'ends ... DONE")
+
+
 parallel::stopCluster(cl)
 
 message("*** plan() ... DONE")
